	local modName = g_currentModName;
	local modDir = g_currentModDirectory;
	ZZZAnywherestorage = {}
	ZZZAnywherestorage_mt = Class(ZZZAnywherestorage, Object)
	ZZZAnywherestorage.sendNumBits = 6

	function onCreate(self,id)
print("created ZZZAnywherestorage, id: ", tostring(id));
	local object = ZZZAnywherestorage:new(g_server ~= nil, g_client ~= nil)
	if object:load(id) then
		g_currentMission:addOnCreateLoadedObject(object)
		object:register(true)
	else
		object:delete()
	end
end
    
    function ZZZAnywherestorage:new(isServer, isClient, customMt)
		local mt = customMt
		if mt == nil then
			mt = ZZZAnywherestorage_mt
		end
		local self = Object:new(isServer, isClient, mt)
			self.ZZZAnywherestorageDirtyFlag = self:getNextDirtyFlag();
			self.className = modName..".ZZZAnywherestorage";
			return self
	end

    function ZZZAnywherestorage:load(id)
    
        self.rootNode = id;
        self.triggerId = Utils.indexToObject(id, getUserAttribute(id, "triggerIndex"));
        if self.triggerId == nil then
            self.triggerId = id;
        end
        addTrigger(self.triggerId, "triggerCallback", self);
  
        self.appearsOnPDA = Utils.getNoNil(getUserAttribute(id, "appearsOnPDA"), false);
        self.isFarmTrigger = Utils.getNoNil(getUserAttribute(id, "isFarmTrigger"), false);
        self.stationName = Utils.getNoNil(getUserAttribute(id, "stationName"), "Station");
    
        self.isAreaTrigger = Utils.getNoNil(getUserAttribute(id, "isAreaTrigger"), false);
        if self.isAreaTrigger then
            self.triggerStartId = getChildAt(id, 0);
            self.triggerEndId = getChildAt(id, 1);
            self.triggerWidth = Utils.getNoNil(getUserAttribute(id, "triggerWidth"), 3);
        else
            self.triggerStartId = self.triggerId;
        end
    
        self.triggerTipWidth = Utils.getNoNil(getUserAttribute(id, "triggerTipWidth"), math.huge);
    
        if self.priceMultipliers == nil or self.acceptedFillTypes == nil then
            self.acceptedFillTypes = {};
            self.priceMultipliers = {};
    
            local fillTypes = getUserAttribute(id, "fillTypes");
            if fillTypes == nil then
                fillTypes = getUserAttribute(id, "fruitTypes");
            end
            local priceMultipliersString = getUserAttribute(id, "priceMultipliers");
            if fillTypes ~= nil then
                local types = Utils.splitString(" ", fillTypes);
    
                local multipliers = {};
                if priceMultipliersString ~= nil then
                    multipliers = Utils.splitString(" ", priceMultipliersString);
                elseif not self.isFarmTrigger and self.defaultPriceMultiplier == nil then
                    print("Error: Missing priceMultipliers string user attribute for ZZZAnywherestorage "..getName(id));
                end;
                for k,v in pairs(types) do
                    local desc = Fillable.fillTypeNameToDesc[v];
                    if desc ~= nil then
                        self.acceptedFillTypes[desc.index] = true;
                        self.priceMultipliers[desc.index] = tonumber(multipliers[k]);
                        if self.priceMultipliers[desc.index] == nil then
                            self.priceMultipliers[desc.index] = Utils.getNoNil(self.defaultPriceMultiplier, 0);
                            if not self.isFarmTrigger and self.defaultPriceMultiplier == nil then
                                print("Error: "..k.."th priceMultiplier is invalid in ZZZAnywherestorage "..getName(id));
                            end;
                        end;
                    end;
                end;
            end;
        end
    
        local movingIndex = getUserAttribute(id, "movingIndex");
        if movingIndex ~= nil then
            local movingRootNode = id;
            if self.triggerId == id then
                movingRootNode = getParent(id);
            end
            self.movingId = Utils.indexToObject(movingRootNode, movingIndex);
            if self.movingId ~= nil then
                self.moveMinY = Utils.getNoNil(getUserAttribute(id, "moveMinY"), 0);
                self.moveMaxY = Utils.getNoNil(getUserAttribute(id, "moveMaxY"), 0);
                self.moveScale = Utils.getNoNil(getUserAttribute(id, "moveScale"), 0.001)*0.01;
                self.moveBackScale = (self.moveMaxY-self.moveMinY)/Utils.getNoNil(getUserAttribute(id, "moveBackTime"), 10000);
            end;
        end;
        
       if self.appearsOnPDA then
           local x, _, z = getWorldTranslation(id);
           local iconSize = g_currentMission.missionPDA.pdaMapWidth / 15;
           self.mapHotspot = g_currentMission.missionPDA:createMapHotspot("TipPlace", "dataS2/missions/hud_pda_spot_tipPlaceGold.png", x, z, iconSize, iconSize * (4 / 3), false, false, false, 0, true);
       end;
   
       self.isEnabled = true;
   
       self.ZZZAnywherestorageDirtyFlag = self:getNextDirtyFlag();
   
   
       return true;
   end;
   
   function ZZZAnywherestorage:delete()
       if self.mapHotspot ~= nil then
           g_currentMission.missionPDA:deleteMapHotspot(self.mapHotspot);
       end;
       
       if self.triggerId ~= 0 then
           removeTrigger(self.triggerId);
       end
       if self.rootNode ~= 0 then
           delete(self.rootNode);
       end
       ZZZAnywherestorage:superClass().delete(self);
   end;
   
   function ZZZAnywherestorage:readStream(streamId, connection)
       if connection:getIsServer() then
           if self.movingId ~= nil then
               local x,y,z = getTranslation(self.movingId);
               y = streamReadFloat32(streamId);
               setTranslation(self.movingId, x, y, z);
           end;
       end;
   end;
   
   function ZZZAnywherestorage:writeStream(streamId, connection)
       if not connection:getIsServer() then
           if self.movingId ~= nil then
               local x,y,z = getTranslation(self.movingId);
               streamWriteFloat32(streamId, y);
           end;
       end;
   end;
   
   function ZZZAnywherestorage:readUpdateStream(streamId, timestamp, connection)
       self:readStream(streamId, connection);
   end;
   
   function ZZZAnywherestorage:writeUpdateStream(streamId, connection, dirtyMask)
       self:writeStream(streamId, connection);
   end;
   
   function ZZZAnywherestorage:update(dt)
       if self.movingId ~= nil then
           local x,y,z = getTranslation(self.movingId);
           local newY = math.max(y-dt*self.moveBackScale, self.moveMinY);
           setTranslation(self.movingId, x, newY, z);
       end;
   end;
   
   function ZZZAnywherestorage:updateMoving(delta)
       if self.movingId ~= nil and self.isServer then
           local x,y,z = getTranslation(self.movingId);
           local newY = math.min(y+delta*self.moveScale, self.moveMaxY);
           setTranslation(self.movingId, x, newY, z);
           self:raiseDirtyFlags(self.ZZZAnywherestorageDirtyFlag);
       end;
   end;
   
   function ZZZAnywherestorage:updateTrailerTipping(trailer, fillDelta, fillType)
       if fillDelta < 0 then
           if self.isFarmTrigger then
               -- put load into storage
               local siloFillType = fillType;
               
			  
				if fillType == Fillable.FILLTYPE_SUGARBEET then
					siloFillType = Fillable.FILLTYPE_RUEBENALTERNATIVE;	
				end;
				if fillType == Fillable.FILLTYPE_POTATO then
					siloFillType = Fillable.FILLTYPE_POTATOALTERNATIVE;	
				end;
               g_currentMission:setSiloAmount(siloFillType, g_currentMission:getSiloAmount(siloFillType)- fillDelta);
					
						
           else
   
               -- update total amount of this fill type
               local desc = Fillable.fillTypeIndexToDesc[fillType];
               desc.totalAmount = desc.totalAmount - fillDelta;
   
               -- increase money according to price of current fill type
               local priceMultiplier = self.priceMultipliers[fillType];
   
               local difficultyMultiplier = math.max(2 * (3 - g_currentMission.missionStats.difficulty), 1); -- 1  2  4
               local greatDemandMultiplier = 1;
   
               -- check if a great demand pertaining to this fill type / station is currently running
               local greatDemand = g_currentMission.economyManager:getCurrentGreatDemand(self.stationName, fillType);
               if greatDemand ~= nil then
                   greatDemandMultiplier = greatDemand.demandMultiplier;
               end;
   
               local money = Fillable.fillTypeIndexToDesc[fillType].pricePerLiter * priceMultiplier * difficultyMultiplier * greatDemandMultiplier * -fillDelta;
               --g_currentMission.missionStats.money = g_currentMission.missionStats.money + money;
               g_currentMission:addSharedMoney(money, "harvestIncome");
           end;
   
           self:updateMoving(-fillDelta);
       end
   end
   
   function ZZZAnywherestorage:getTipInfoForTrailer(trailer, tipReferencePointIndex)
       local minDistance, bestPoint = self:getTipDistanceFromTrailer(trailer, tipReferencePointIndex);
   
       local isAllowed = false;
       if self.acceptedFillTypes[trailer.currentFillType] then
           isAllowed = true;
       end
   
       return isAllowed, minDistance, bestPoint;
   end
   
   function ZZZAnywherestorage:getTipDistanceFromTrailer(trailer, tipReferencePointIndex)
   
       local minDistance = math.huge;
       local bestPoint = nil;
       local bestWidth = -1;
   
       local sx, _, sz = getWorldTranslation(self.triggerStartId);
       if self.isAreaTrigger then
           local ex, _, ez = getWorldTranslation(self.triggerEndId);
           local dx, dz = ex-sx, ez-sz;
           local length = math.sqrt(dx*dx + dz*dz);
           if length > 0.00001 then
               dx = dx / length;
               dz = dz / length;
           end
           if tipReferencePointIndex ~= nil then
               local trailerX, _, trailerZ = getWorldTranslation(trailer.tipReferencePoints[tipReferencePointIndex].node);
               return Utils.getDistanceToRectangle2D(trailerX, trailerZ, sx, sz, dx, dz, length, self.triggerWidth), tipReferencePointIndex;
           else
               for i,point in pairs(trailer.tipReferencePoints) do
                   local trailerX, _, trailerZ = getWorldTranslation(point.node);
                   local distance = Utils.getDistanceToRectangle2D(trailerX, trailerZ, sx, sz, dx, dz, length, self.triggerWidth);
                   if point.width <= self.triggerTipWidth and point.width > bestWidth then
                       -- if the tip point matches the trigger better, then also accept it if the distance does not improve
                       if distance <= minDistance+0.001 then
                           bestPoint = i;
                           bestWidth = point.width;
                           minDistance = distance;
                       end
                   else
                       if distance < minDistance then
                           bestPoint = i;
                           minDistance = distance;
                       end
                   end
               end
           end
       else
           if tipReferencePointIndex ~= nil then
               local trailerX, _, trailerZ = getWorldTranslation(trailer.tipReferencePoints[tipReferencePointIndex].node);
               return Utils.vector2Length(trailerX-sx, trailerZ-sz), tipReferencePointIndex;
           else
               for i,point in pairs(trailer.tipReferencePoints) do
                   local trailerX, _, trailerZ = getWorldTranslation(point.node);
                   local distance = Utils.vector2Length(trailerX-sx, trailerZ-sz);
                   if point.width <= self.triggerTipWidth and point.width > bestWidth then
                       -- if the tip point matches the trigger better, then also accept it if the distance does not improve
                       if distance <= minDistance+0.001 then
                           bestPoint = i;
                           bestWidth = point.width;
                           minDistance = distance;
                       end
                   else
                       if distance < minDistance then
                           bestPoint = i;
                           minDistance = distance;
                       end
                   end
               end
           end
       end
       return minDistance, bestPoint;
   end;
   
   function ZZZAnywherestorage:getNoAllowedText(trailer)
       if trailer.currentFillType ~= Fillable.FILLTYPE_UNKNOWN and not self.acceptedFillTypes[trailer.currentFillType] then
           return Fillable.fillTypeIndexToDesc[trailer.currentFillType].nameI18N .. g_i18n:getText("notAcceptedHere");
       end
       return nil;
   end
   
   function ZZZAnywherestorage:triggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
   
       if self.isEnabled then
           if onEnter then
               local trailer = g_currentMission.objectToTrailer[otherShapeId];
               if trailer ~= nil and trailer.allowTipDischarge then
                   if g_currentMission.trailerTipTriggers[trailer] == nil then
                       g_currentMission.trailerTipTriggers[trailer] = {};
                   end;
                   table.insert(g_currentMission.trailerTipTriggers[trailer], self);
               end;
           elseif onLeave then
               local trailer = g_currentMission.objectToTrailer[otherShapeId];
               if trailer ~= nil and trailer.allowTipDischarge then
                   local triggers = g_currentMission.trailerTipTriggers[trailer];
                   if triggers ~= nil then
                       for i=1, table.getn(triggers) do
                           if triggers[i] == self then
                               table.remove(triggers, i);
                               if table.getn(triggers) == 0 then
                                   g_currentMission.trailerTipTriggers[trailer] = nil;
                               end;
                               break;
                           end;
                       end;
                   end;
               end;
           end;
       end;
   end;